@@ -15,7 +15,7 @@
 #' retrieval: theory and practice" (1979) and includes 250 words.
 #' `text2map`'s very own stoplist `tiny2020` is a lean 34 words.
 #'
-#' Below are stoplists available with [get_stoplist()]:
+#' Below are stoplists available with [get_stoplist]:
 #' - "tiny2020": Tiny (2020) list of 33 words (Default)
 #' - "snowball2001": Snowball stemming package's (2001) list of 127 words
 #' - "snowball2014": Updated Snowball (2014) list of 175 words
@@ -47,6 +47,8 @@
 #'
 #' @export
 #'
+#'
+#'
 get_stoplist <- function(source = "tiny2020", language = "en", tidy = FALSE) {
   if (source %in% c(
     "van1979", "fox1990",
@@ -128,7 +130,7 @@ tiny_gender_tagger <- function() {
 #' Returns the elements of `x` that are "in" `y`, uses `fastmatch`'s
 #' matching backend.
 #'
-#'
+#' @import fastmatch
 #' @importFrom fastmatch %fin%
 #' @importFrom fastmatch fmatch
 #'
@@ -142,7 +144,7 @@ tiny_gender_tagger <- function() {
 #' @author Dustin Stoltz
 #'
 #' @noRd
-`%in%` <- `%fin%`
+`%in%` <- fastmatch::`%fin%`
 
 
 #' Fast-not-match `%fnin%` operator
@@ -164,7 +166,7 @@ tiny_gender_tagger <- function() {
 #' @author Dustin Stoltz
 #'
 #' @noRd
-`%fnin%` <- Negate(`%fin%`)
+`%fnin%` <- Negate(fastmatch::`%fin%`)
 
 
 #' .kurtosis
@@ -183,7 +185,7 @@ tiny_gender_tagger <- function() {
   return(out)
 }
 
-#' .kurtosis
+#' .skewness
 #'
 #' adapted from e1071 package type=3 in the skewness function:
 #' b_1 = m_3/s^3 = g_((n - 1)/n)^3/2
@@ -220,6 +222,20 @@ tiny_gender_tagger <- function() {
   }
 }
 
+#' .check_whole_num
+#'
+#' Checks if numbers are integers/whole numbers
+#'
+#' @param x a vector of numbers
+#' @param any logical (default FALSE), return any TRUE
+#'
+#' @noRd
+.check_whole_num <- function(x) {
+  out <- is.integer(x)
+
+  return(isTRUE(out))
+}
+
 #' quiets concerns of R CMD check re: i in loops and lapply
 #'
 #' @noRd
