@@ -78,8 +78,8 @@ test_that("dtm_melter works on both base and sparse", {
     expect_equal(ncol(out_a), as.integer(3L))
     expect_equal(ncol(out_b), as.integer(3L))
     expect_equal(out_a, out_b)
-    expect_equal(sum(out_a$count), sum(dtm.bse))
-    expect_equal(sum(out_b$count), sum(dtm.dgc))
+    expect_equal(sum(out_a$freq), sum(dtm.bse))
+    expect_equal(sum(out_b$freq), sum(dtm.dgc))
 
 })
 
@@ -88,14 +88,14 @@ test_that("dtm_builder produces identical dtm to cast_dtm", {
 
     my.corpus <- data.frame(
         text = c("I hear babies crying I watch them grow",
-                 "They’ll learn much more than I'll ever know",
+                 "They'll learn much more than I'll ever know",
                  "And I think to myself",
                  "What a wonderful world",
                  "Yes I think to myself",
                  "What a wonderful world"),
-        line_id = paste0("line", 1:6))
+        line_id = paste0("line", seq_len(6)))
     # some text preprocessing
-    my.corpus$clean_text <- tolower(gsub("'|’", "", my.corpus$text) )
+    my.corpus$clean_text <- tolower(gsub("'", "", my.corpus$text))
 
     # example 1
     dtm.a <- my.corpus %>%
@@ -106,7 +106,7 @@ test_that("dtm_builder produces identical dtm to cast_dtm", {
 
     # example 3
     dtm.c <- my.corpus %>%
-        dplyr::mutate(clean_text = gsub("'|’", "", text),
+        dplyr::mutate(clean_text = gsub("'", "", text),
                       clean_text = tolower(clean_text)) %>%
         dtm_builder(clean_text, line_id)
 
@@ -147,7 +147,7 @@ test_that("dtm_builder error/message if last row is blank", {
                     "What a wonderful world",
                     "Yes I think to myself",
                     "What a wonderful world"),
-        line_id = paste0("line", 1:6))
+        line_id = paste0("line", seq_len(6)))
 
     my.corpus$my_text[6] <- ""
 
@@ -156,6 +156,7 @@ test_that("dtm_builder error/message if last row is blank", {
 
 })
 
+
 test_that("dtm_builder works with vocab", {
 
     vocab <- vocab_builder(corpus, text)
@@ -181,17 +182,17 @@ test_that("dtm_builder works with vocab", {
 
     expect_error(
         expect_message(
-            dtm_builder(corpus, text, doc_ids,
-                        vocab = c("hear", "babies",
-                                  "world", "picklespit"))))
+        dtm_builder(corpus, text, doc_ids,
+                    vocab = c("hear", "babies",
+                              "world", "picklespit"))))
     expect_error(
         expect_message(
-            dtm_builder(corpus, text, doc_ids,
-                        vocab = new.vocab, chunk=5L) ))
+        dtm_builder(corpus, text, doc_ids,
+                    vocab = new.vocab, chunk=5L) ))
     expect_error(
         expect_message(
-            dtm_builder(corpus, text, doc_ids,
-                        vocab = new.vocab) ))
+        dtm_builder(corpus, text, doc_ids,
+                    vocab = new.vocab) ))
 })
 
 test_that("dtm_builder error/message if doc_id is wrong...", {
@@ -203,7 +204,7 @@ test_that("dtm_builder error/message if doc_id is wrong...", {
                     "What a wonderful world",
                     "Yes I think to myself",
                     "What a wonderful world"),
-        line_id = paste0("line", 1:6))
+        line_id = paste0("line", seq_len(6)))
 
     expect_error(
         expect_message(
@@ -231,7 +232,6 @@ test_that("dtm_builder chunks correctly", {
 
 test_that("dtm resampler creates DTM of the same dimensions", {
 
-
     out <- dtm_resampler(dtm.dgc, alpha=0.2)
     expect_s4_class(out, "dgCMatrix")
     expect_identical(dim(out), dim(dtm.dgc))
@@ -265,49 +265,138 @@ test_that("dtm resampler creates DTM of the same dimensions", {
 
 })
 
-
-test_that("dtm_stopper returns expected output", {
-
-    # example 1
-    dtm.a <- corpus %>%
-        dtm_builder(text, doc_id)
-    dtm.a <- cbind(dtm.a, as.matrix(empty <- rep(0, nrow(dtm.a))) )
-    dtm.a <- cbind(dtm.a, as.matrix(rep(0, nrow(dtm.a))))
+test_that("dtm_stopper works with stop_list", {
 
     expect_identical(
-        dim(dtm_stopper(dtm.a, stop_list = c("choose", "moon"))),
-        as.integer(c(10, 44)))
+        dim(dtm_stopper(dtm.dgc,
+            stop_list = c("we", "moon"))),
+        as.integer(c(10, 41)))
     expect_identical(
-        dim(dtm_stopper(dtm.a, stop_list = c("We", "we", "moon"), ignore_case = FALSE)),
-        as.integer(c(10, 43)))
+        dim(dtm_stopper(dtm.dgc,
+            stop_list = c("We", "we", "moon"),
+            ignore_case = FALSE)),
+        as.integer(c(10, 41)))
     expect_identical(
-        dim(dtm_stopper(dtm.a, stop_termfreq = c(2L, 5L)) ),
+        dim(dtm_stopper(dtm.dgc,
+             stop_list = c("we", "moon"),
+             ignore_case = FALSE)),
+        as.integer(c(10, 42)))
+
+})
+
+test_that("dtm_stopper works with stop_termfreq", {
+
+    expect_identical(
+        dim(dtm_stopper(dtm.dgc, stop_termfreq = c(2L, 5L))),
         as.integer(c(10, 12)))
     expect_identical(
-        dim(dtm_stopper(dtm.a, stop_termfreq = c(1L, 2L) )),
+        dim(dtm_stopper(dtm.dgc, stop_termfreq = c(1L, 2L))),
         as.integer(c(10, 35)))
     expect_identical(
-        dim(dtm_stopper(dtm.a, stop_termfreq = c(0.04, 0.99) )),
+        dim(dtm_stopper(dtm.dgc, stop_termfreq = c(1, 2))),
+        as.integer(c(10, 35)))
+})
+
+test_that("dtm_stopper works with stop_termprop", {
+    expect_identical(
+        dim(dtm_stopper(dtm.dgc, stop_termprop = c(0.04, 0.99))),
         as.integer(c(10, 7)))
     expect_identical(
-        dim(dtm_stopper(dtm.a, stop_docfreq = c(0.2, 0.99) )),
+        dim(dtm_stopper(dtm.dgc, stop_termprop = c(0.01, 0.1))),
+        as.integer(c(10, 43)))
+
+})
+
+test_that("dtm_stopper works with stop_docprop", {
+
+    expect_identical(
+        dim(dtm_stopper(dtm.dgc, stop_docfreq = c(1L, 3L))),
+        as.integer(c(10, 39)))
+    expect_identical(
+        dim(dtm_stopper(dtm.dgc, stop_docfreq = c(2L, 4L))),
+        as.integer(c(10, 12)))
+})
+
+test_that("dtm_stopper works with stop_docprop", {
+
+    expect_identical(
+        dim(dtm_stopper(dtm.dgc, stop_docprop = c(0.2, .98))),
         as.integer(c(10, 13)))
     expect_identical(
-        dim(dtm_stopper(dtm.a, stop_docfreq = c(0.1, 0.4) )),
+        dim(dtm_stopper(dtm.dgc, stop_docprop = c(0.1, 0.4))),
         as.integer(c(10, 43)))
+})
+
+test_that("dtm_stopper works with Inf", {
+
+    # docprop
     expect_identical(
-        dim(dtm_stopper(dtm.a, stop_docfreq = c(1L, 3L) )),
+        dim(dtm_stopper(dtm.dgc, stop_docprop = c(0.2, Inf))),
+        as.integer(c(10, 13)))
+    expect_identical(
+        dim(dtm_stopper(dtm.dgc, stop_docprop = c(Inf, 1.0))),
+        as.integer(c(10, 44)))
+    # docfreq
+    expect_identical(
+        dim(dtm_stopper(dtm.dgc, stop_docfreq = c(1L, 3L))),
         as.integer(c(10, 39)))
+    #termfreq
+    expect_identical(
+        dim(dtm_stopper(dtm.dgc, stop_termfreq = c(Inf, 2))),
+        as.integer(c(10, 35)))
+    #termprop
+    expect_identical(
+        dim(dtm_stopper(dtm.dgc, stop_termprop = c(0.04, Inf))),
+        as.integer(c(10, 7)))
+})
+
+test_that("dtm_stopper works with happax and null", {
+
+    # add empty column to dtm
+    mat <- as.matrix(rep(0, nrow(dtm.dgc)))
+    colnames(mat) <- "empty"
+    dtm.a <- cbind(dtm.dgc, mat)
+
     expect_identical(
         dim(dtm_stopper(dtm.a, stop_hapax = TRUE)),
         as.integer(c(10, 13)))
+
+    # should remove the one null column
     expect_identical(
         dim(dtm_stopper(dtm.a, stop_null = TRUE)),
-        as.integer(c(10, 46)))
-    expect_error(expect_message(dtm_stopper(dtm.a, stop_termfreq = c("picklespit")) ))
-    expect_error(expect_message(dtm_stopper(dtm.a, stop_docfreq = c("picklespit")) ))
-    expect_error(expect_message(dtm_stopper(dtm.a)))
-    expect_error(expect_message(dtm_stopper(as.matrix(dtm.a))))
+        as.integer(c(10, 44)))
+
+})
+
+test_that("dtm_stopper works with stoprank", {
+
+    expect_identical(
+        dim(dtm_stopper(dtm.dgc, stop_termrank = 2L)),
+        as.integer(c(10, 42)))
+
+    expect_identical(
+        dim(dtm_stopper(dtm.dgc, stop_termrank = 5L)),
+        as.integer(c(10, 39)))
+
+    expect_identical(
+        dim(dtm_stopper(dtm.dgc, stop_termrank = 20L)),
+        as.integer(c(10, 24)))
+
+})
+
+
+test_that("dtm_stopper errors work", {
+
+expect_error(
+      expect_message(dtm_stopper(dtm.dgc, stop_termfreq = c("picklespit"))))
+expect_error(
+      expect_message(dtm_stopper(dtm.dgc, stop_docfreq = c("picklespit"))))
+expect_error(
+      expect_message(dtm_stopper(dtm.dgc)))
+expect_error(
+      expect_message(dtm_stopper(as.matrix(dtm.dgc))))
+expect_error(
+    expect_message(dtm_stopper(dtm.dgc, stop_termrank = "picklespit")))
 
 })
 
@@ -330,15 +419,80 @@ test_that(".convert_dtm_to_dgCMatrix convert all dtms to dgCMatrix", {
 test_that("dtm resampler works on output of .prep_cmd_INPUT", {
 
     ## dgCMatrix matrix ##
-    out <- .prep_cmd_INPUT(dtm=dtm.dgc[,1:35],
+    out <- .prep_cmd_INPUT(dtm=dtm.dgc[, seq_len(35)],
                             cw = cw.oov,
                             cv = NULL,
                             wv = fake_word_vectors_oov,
                             missing = "stop")
 
-    dtm.samp <- dtm_resampler(out$DTM, alpha=1)
+    dtm.samp <- dtm_resampler(out$DTM, alpha = 1)
     expect_s4_class(dtm.samp, "dgCMatrix")
     expect_equal(nrow(dtm.samp), nrow(out$DTM))
     expect_equal(ncol(dtm.samp), ncol(out$DTM))
 
 })
+
+test_that("compare dtm dense and sparse", {
+
+    dtm_sp <- corpus %>%
+        dtm_builder(text, doc_id, dense = FALSE)
+
+    dtm_de <- corpus %>%
+        dtm_builder(text, doc_id, dense = TRUE)
+
+    expect_s4_class(dtm_sp, "dgCMatrix")
+    expect_identical(class(dtm_de),
+                     c("matrix", "array"))
+    expect_equal(nrow(dtm_sp), nrow(dtm_de))
+    expect_equal(ncol(dtm_sp), ncol(dtm_de))
+
+    # compare with vocab
+    vocab <- vocab_builder(corpus, text)
+    new.vocab <- vocab[!vocab %in% c("moon")]
+
+    dtm_sp  <- dtm_builder(corpus, text, doc_id,
+                           vocab = new.vocab,
+                           dense = FALSE)
+
+    dtm_de  <- dtm_builder(corpus, text, doc_id,
+                           vocab = new.vocab,
+                           dense = TRUE)
+
+    expect_s4_class(dtm_sp, "dgCMatrix")
+    expect_identical(class(dtm_de),
+                     c("matrix", "array"))
+    expect_equal(nrow(dtm_sp), nrow(dtm_de))
+    expect_equal(ncol(dtm_sp), ncol(dtm_de))
+
+})
+
+test_that("works w/o doc_id", {
+
+    dtm1 <- corpus %>%
+        dtm_builder(text, doc_id, dense = FALSE)
+    dtm2 <- corpus %>%
+        dtm_builder(text, dense = FALSE)
+
+    expect_equal(nrow(dtm1), nrow(dtm2))
+    expect_equal(ncol(dtm1), ncol(dtm2))
+
+})
+
+test_that("dtm functions with one document", {
+
+    dtm1 <- dtm_builder(corpus[1, ], text, doc_id, dense = FALSE)
+    expect_identical(dim(dtm1), as.integer(c(1, 6)))
+
+    dtm2 <- dtm_stopper(dtm1, stop_list = c("we", "moon"))
+    expect_identical(dim(dtm2), as.integer(c(1, 4)))
+
+    expect_identical(dim(dtm_melter(dtm1)),
+                     as.integer(c(6, 3)))
+
+    dtm_c <- dtm_builder(corpus[1, ], text, chunk = 1L, dense = FALSE)
+    expect_identical(dim(dtm_c), as.integer(c(7, 6)))
+
+    expect_identical(dim(dtm_melter(dtm_c)),
+                     as.integer(c(7, 3)))
+
+})
